/***************************************************************************
                          kountdown.cpp  -  description
                             -------------------
    begin                : Sun Feb 10 09:52:20 CET 2002
    copyright            : (C) 2002 by Erik Johansson
    email                : erre@telia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define DEBUG

#include <qcolor.h>
#include <qstring.h>
#include <qpainter.h>
#include <qdatetime.h>
#include <qpixmap.h>
#include <qtextstream.h>
#include <qfont.h>

#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>

#include "kountdown.h"
#include "timer.h"
#include "global.h"

Kountdown::Kountdown(WId id)
	: KScreenSaver (id), repaintScreen(true)
{
	readGlobalSettings();
	readSettings(0);
	qpixmap = 0;
	kountdown();
}

Kountdown::~Kountdown()
{
	killTimer( timers );
}

void Kountdown::readGlobalSettings()
{
	int startFor, halfFor, startRound, halfRound;
	KConfig *config = KGlobal::config();
	config->setGroup("Global settings");

	QDateTime temp( QDate::currentDate().addDays(10), QTime::currentTime() );
	QDateTime finish = config->readDateTimeEntry( "FinishDateTime", &temp);
	QDateTime *half = new QDateTime;
	
	if ( config->readBoolEntry( "UseHalfWay", false ) )
	{
		temp = QDateTime( QDate::currentDate().addDays(5), QTime::currentTime() );
   	*half = config->readDateTimeEntry( "HalfDateTime", &temp);
	} else {
		delete half;
		half = 0;
	}
	
	config->setGroup( "Start settings" );
	startFor = config->readNumEntry( "Display", 15 );
	startRound = config->readNumEntry( "Round Off", 0 );
	
	config->setGroup( "HalfWay settings" );
	halfFor = config->readNumEntry( "Display", 15 );
	halfRound = config->readNumEntry( "Round Off", 0 );

	timer = new Timer( &finish, startFor, startRound, half, halfFor, halfRound );
	CHECK_PTR( timer );
	connect(timer, SIGNAL( changeSettings(int) ), this, SLOT( readSettings(int) ));
}

void Kountdown::readSettings( int moment )
{
	KConfig *config = KGlobal::config();

	switch (moment)
	{
	case 0:
		config->setGroup( "Start settings" );
		break;
	case 1:
		config->setGroup( "HalfWay settings" );
		break;
	case 2:
		config->setGroup( "Finish settings" );
		break;
	}

	text = config->readEntry( "Text", i18n( "Default text" ));
	backCol = config->readColorEntry( "BackCol", &Qt::black );
	fontCol = config->readColorEntry( "FontCol", &Qt::blue );
	font = config->readFontEntry( "Font" );
}

void Kountdown::kountdown()
{
	erase();
	timers = startTimer(300);
}

void Kountdown::timerEvent(QTimerEvent *)
{
	STimeLeft tiden;
	tiden = timer->getTime();
	if ( tiden.error )
		textDisplay = QString( "Error in format. Check file: %1, line: %2" ).arg(__FILE__).arg(__LINE__);
	else
	{
		textDisplay = text;
		textDisplay.replace( QRegExp( "%[Dd]" ), QString::number(tiden.days) );
		textDisplay.replace( QRegExp( "%[Hh]" ), QString::number(tiden.hours) );
		textDisplay.replace( QRegExp( "%[Mm]" ), QString::number(tiden.mins) );
		textDisplay.replace( QRegExp( "%[Ss]" ), QString::number(tiden.secs) );
	}

	if ( textDisplay != oldTimeString )
	{
		repaintScreen = true;
		oldTimeString = textDisplay;
	}
	repaint(false);
}

void Kountdown::paintEvent( QPaintEvent * )
{
	if ( repaintScreen )
	{
		if ( qpixmap != 0 )
			delete qpixmap;
		qpixmap = new QPixmap (width(), height());

		QPainter paint(qpixmap);
		paint.fillRect( qpixmap->rect(), backCol );
		paint.setPen( fontCol );
		paint.setFont( font );
		paint.drawText( qpixmap->rect(), AlignCenter, textDisplay );
	}
	repaintScreen = false;
	bitBlt( this, 0, 0, qpixmap, 0, 0, -1, -1, CopyROP);
}
